@@ -115,8 +115,8 @@ static int fontset_width __ARGS((XFontSet fs));
 static int fontset_ascent __ARGS((XFontSet fs));
 #endif
 
-static guicolor_T	prev_fg_color = (guicolor_T)-1;
-static guicolor_T	prev_bg_color = (guicolor_T)-1;
+static guicolor_T	prev_fg_color = INVALCOLOR;
+static guicolor_T	prev_bg_color = INVALCOLOR;
 
 #if defined(FEAT_GUI_MOTIF) && defined(FEAT_MENU)
 static XButtonPressedEvent last_mouse_event;
@@ -143,6 +143,11 @@ static void gui_x11_blink_cb __ARGS((XtPointer timed_out, XtIntervalId *interval
 static Cursor gui_x11_create_blank_mouse __ARGS((void));
 
 
+/*
+ * Keycodes recognized by vim.
+ * NOTE: when changing this, the table in gui_gtk_x11.c probably needs the
+ * same change!
+ */
 static struct specialkey
 {
     KeySym  key_sym;
@@ -215,12 +220,12 @@ static struct specialkey
     {XK_KP_Right,	'k', 'r'},
     {XK_KP_Up,		'k', 'u'},
     {XK_KP_Down,	'k', 'd'},
-    {XK_KP_Insert,	'k', 'I'},
-    {XK_KP_Delete,	'k', 'D'},
-    {XK_KP_Home,	'k', 'h'},
-    {XK_KP_End,		'@', '7'},
-    {XK_KP_Prior,	'k', 'P'},
-    {XK_KP_Next,	'k', 'N'},
+    {XK_KP_Insert,	KS_EXTRA, (char_u)KE_KINS},
+    {XK_KP_Delete,	KS_EXTRA, (char_u)KE_KDEL},
+    {XK_KP_Home,	'K', '1'},
+    {XK_KP_End,		'K', '4'},
+    {XK_KP_Prior,	'K', '3'},
+    {XK_KP_Next,	'K', '5'},
 
     {XK_KP_Add,		'K', '6'},
     {XK_KP_Subtract,	'K', '7'},
@@ -651,6 +656,15 @@ gui_x11_resize_window_cb(w, dud, event, dum)
 	workshop_frame_moved(rec.x, rec.y, rec.width, rec.height);
     }
 #endif
+#ifdef FEAT_NETBEANS_INTG
+    if (usingNetbeans)
+    {
+	XRectangle  rec;
+
+	shellRectangle(w, &rec);
+	netbeans_frame_moved(rec.x, rec.y);
+    }
+#endif
 #ifdef FEAT_XIM
     xim_set_preedit();
 #endif
@@ -780,7 +794,7 @@ gui_x11_key_hit_cb(w, dud, event, dum)
 	    {
 		char_u	*p = (char_u *)XtMalloc(len + 3);
 
-		mch_memmove(p, string, i);
+		mch_memmove(p, string, i + 1);
 		p[i + 1] = KS_EXTRA;
 		p[i + 2] = (int)KE_CSI;
 		mch_memmove(p + i + 3, string + i + 1, len - i);
@@ -789,6 +803,7 @@ gui_x11_key_hit_cb(w, dud, event, dum)
 		string = p;
 		string_alloced = True;
 		i += 2;
+		len += 2;
 	    }
     }
     else
@@ -1162,6 +1177,17 @@ gui_mch_prepare(argc, argv)
 # endif
 	}
 	else
+#endif
+#ifdef FEAT_NETBEANS_INTG
+	    if (strncmp("-nb", argv[arg], 3) == 0)
+	{
+	    usingNetbeans++;
+	    gui.dofork = FALSE;	/* don't fork() when starting GUI */
+	    netbeansArg = argv[arg];
+	    mch_memmove(&argv[arg], &argv[arg + 1],
+					    (--*argc - arg) * sizeof(char *));
+	}
+	else
 #endif
 	    arg++;
     }
@@ -1422,13 +1448,17 @@ gui_mch_init()
     if (usingSunWorkShop)
 	workshop_connect(app_context);
 #endif
+#ifdef FEAT_NETBEANS_INTG
+    if (usingNetbeans)
+	netbeans_Xt_connect(app_context);
+#endif
 
-# ifdef FEAT_BEVAL
+#ifdef FEAT_BEVAL
     gui_init_tooltip_font();
-# endif
-# ifdef FEAT_MENU
+#endif
+#ifdef FEAT_MENU
     gui_init_menu_font();
-# endif
+#endif
 
     return OK;
 }
@@ -2122,7 +2152,7 @@ fontset_ascent(fs)
 
 /*
  * Return the Pixel value (color) for the given color name.
- * Return -1 for error.
+ * Return INVALCOLOR for error.
  */
     guicolor_T
 gui_mch_get_color(reqname)
@@ -2149,7 +2179,7 @@ gui_mch_get_color(reqname)
 
     /* can't do this when GUI not running */
     if (!gui.in_use || *reqname == NUL)
-	return (guicolor_T)-1;
+	return INVALCOLOR;
 
     colormap = DefaultColormap(gui.dpy, XDefaultScreen(gui.dpy));
 
@@ -2201,7 +2231,7 @@ gui_mch_get_color(reqname)
 	}
     }
 
-    return (guicolor_T)-1;
+    return INVALCOLOR;
 }
 
 /*
@@ -2397,7 +2427,7 @@ gui_mch_draw_string(row, col, s, len, flags)
 	    XDrawString(gui.dpy, gui.wid, gui.text_gc, TEXT_X(col),
 		    TEXT_Y(row), (char *)s, len);
     }
-    else if (p_linespace
+    else if (p_linespace != 0
 #ifdef FEAT_XFONTSET
 	    || current_fontset != NULL
 #endif
@@ -2573,8 +2603,8 @@ gui_mch_draw_part_cursor(w, h, color)
 	    !(State & CMDLINE) && curwin->w_p_rl ? FILL_X(gui.col + 1) - w :
 #endif
 		FILL_X(gui.col),
-	    FILL_Y(gui.row) + gui.char_height - h + (int)p_linespace / 2,
-	    w, h - (int)p_linespace);
+	    FILL_Y(gui.row) + gui.char_height - h,
+	    w, h);
 }
 
 /*
@@ -3225,7 +3255,7 @@ gui_mch_register_sign(signfile)
 	{
 	    XpmColorSymbol color[5] =
 	    {
-		{"none", "none", 0},
+		{"none", NULL, 0},
 		{"iconColor1", NULL, 0},
 		{"bottomShadowColor", NULL, 0},
 		{"topShadowColor", NULL, 0},
@@ -3251,7 +3281,6 @@ gui_mch_register_sign(signfile)
 		sign = NULL;
 		EMSG(_("E255: Couldn't read in sign data!"));
 	    }
-	    vim_free(attrs.colorsymbols);
 	}
     }
 
@@ -3447,13 +3476,12 @@ get_toolbar_pixmap(menu, sen, insen)
 
     if (menu->iconfile != NULL)
     {
-	/* Use the file argument: first as an absolute path (with extension),
-	 * then as a file name (without extension). */
-	expand_env(menu->iconfile, buf, MAXPATHL);
+	/* Use the "icon="  argument. */
+	gui_find_iconfile(menu->iconfile, buf, "xpm");
 	createXpmImages(buf, NULL, sen, insen);
-	if (*sen == (Pixmap)0
-		&& gui_find_bitmap(menu->name, buf, "xpm") == OK
-		&& buf[0] != NUL)
+
+	/* If it failed, try using the menu name. */
+	if (*sen == (Pixmap)0 && gui_find_bitmap(menu->name, buf, "xpm") == OK)
 	    createXpmImages(buf, NULL, sen, insen);
 	if (*sen != (Pixmap)0)
 	    return;
