@@ -22,6 +22,13 @@
 # include <fcntl.h>
 #endif
 
+#ifdef __CYGWIN__
+# ifndef WIN32
+#  include <sys/cygwin.h>	/* for cygwin_conv_to_posix_path() */
+# endif
+# include <limits.h>
+#endif
+
 #if defined(UNIX) || defined(VMS)
 static int file_owned __ARGS((char *fname));
 #endif
@@ -140,6 +147,11 @@ main
     int		literal = FALSE;	/* don't expand file names */
 #endif
 
+# ifdef NBDEBUG
+    nbdebug_wait(WT_ENV | WT_WAIT | WT_STOP, "SPRO_GVIM_WAIT", 20);
+    nbdebug_log_init("SPRO_GVIM_DEBUG", "SPRO_GVIM_DLEVEL");
+# endif
+
     /*
      * Do any system-specific initialisations.  These can NOT use IObuff or
      * NameBuff.  Thus emsg2() cannot be called!
@@ -147,7 +159,7 @@ main
     mch_early_init();
 
 #ifdef FEAT_TCL
-    tcl_init(argv[0]);
+    vim_tcl_init(argv[0]);
 #endif
 
 #ifdef MEM_PROFILE
@@ -301,6 +313,9 @@ main
 #ifdef FEAT_SUN_WORKSHOP
     findYourself(argv[0]);
 #endif
+#ifdef FEAT_NETBEANS_INTG
+    netbeans_setRunDir(argv[0]);
+#endif
 #if defined(FEAT_GUI) && !defined(MAC_OS_CLASSIC)
     gui_prepare(&argc, argv);	/* Prepare for possibly starting GUI sometime */
     TIME_MSG("GUI prepared");
@@ -406,15 +421,16 @@ main
     /* TODO: On MacOS X default to gui if argv[0] ends in:
      *       /vim.app/Contents/MacOS/Vim */
 
-    if (TO_LOWER(initstr[0]) == 'r')
+    if (TOLOWER_ASC(initstr[0]) == 'r')
     {
 	restricted = TRUE;
 	++initstr;
     }
 
     /* Avoid using evim mode for "editor". */
-    if (TO_LOWER(initstr[0]) == 'e'
-	    && (TO_LOWER(initstr[1]) == 'v' || TO_LOWER(initstr[1]) == 'g'))
+    if (TOLOWER_ASC(initstr[0]) == 'e'
+	    && (TOLOWER_ASC(initstr[1]) == 'v'
+					   || TOLOWER_ASC(initstr[1]) == 'g'))
     {
 #ifdef FEAT_GUI
 	gui.starting = TRUE;
@@ -423,7 +439,7 @@ main
 	++initstr;
     }
 
-    if (TO_LOWER(initstr[0]) == 'g')
+    if (TOLOWER_ASC(initstr[0]) == 'g')
     {
 	main_start_gui();
 #ifdef FEAT_GUI
@@ -501,10 +517,10 @@ main
 	    if (c == '/')
 	    {
 		c = argv[0][argv_idx++];
-		c = TO_UPPER(c);
+		c = TOUPPER_ASC(c);
 	    }
 	    else
-		c = TO_LOWER(c);
+		c = TOLOWER_ASC(c);
 #endif
 	    switch (c)
 	    {
@@ -535,7 +551,7 @@ main
 		    Columns = 80;	/* need to init Columns */
 		    info_message = TRUE; /* use mch_msg(), not mch_errmsg() */
 		    list_version();
-		    mch_exit(1);
+		    mch_exit(0);
 		}
 		else if (STRNICMP(argv[0] + argv_idx, "literal", 7) == 0)
 		{
@@ -824,7 +840,7 @@ main
 		    mainerr(ME_GARBAGE, (char_u *)argv[0]);
 
 		--argc;
-		if (argc < 1)
+		if (argc < 1 && c != 'S')
 		    mainerr_arg_missing((char_u *)argv[0]);
 		++argv;
 		argv_idx = -1;
@@ -837,10 +853,26 @@ main
 			mainerr(ME_EXTRA_CMD, NULL);
 		    if (c == 'S')
 		    {
-			p = alloc((unsigned)(STRLEN(argv[0]) + 4));
+			char	*a;
+
+			if (argc < 1)
+			    /* "-S" without argument: use default session file
+			     * name. */
+			    a = SESSION_FILE;
+			else if (argv[0][0] == '-')
+			{
+			    /* "-S" followed by another option: use default
+			     * session file name. */
+			    a = SESSION_FILE;
+			    ++argc;
+			    --argv;
+			}
+			else
+			    a = argv[0];
+			p = alloc((unsigned)(STRLEN(a) + 4));
 			if (p == NULL)
 			    mch_exit(2);
-			sprintf((char *)p, "so %s", argv[0]);
+			sprintf((char *)p, "so %s", a);
 			commands[n_commands++] = p;
 		    }
 		    else
@@ -971,6 +1003,24 @@ scripterror:
 		    p = r;
 		}
 	    }
+#endif
+#if defined(__CYGWIN32__) && !defined(WIN32)
+            /*
+             * If vim is invoked by non-Cygwin tools, convert away any
+             * DOS paths, so things like .swp files are created correctly.
+             * Look for evidence of non-Cygwin paths before we bother.
+	     * This is only for when using the Unix files.
+             */
+            if (strpbrk(p, "\\:") != NULL)
+	    {
+                char posix_path[PATH_MAX];
+
+                cygwin_conv_to_posix_path(p, posix_path);
+                vim_free(p);
+                p = vim_strsave(posix_path);
+                if (p == NULL)
+                    mch_exit(2);
+            }
 #endif
 	    alist_add(&global_alist, p,
 #if (!defined(UNIX) && !defined(__EMX__)) || defined(ARCHIE)
@@ -1119,7 +1169,8 @@ scripterror:
 	if (!input_isatty)
 	    mch_errmsg(_("Vim: Warning: Input is not from a terminal\n"));
 	out_flush();
-	ui_delay(2000L, TRUE);
+	if (scriptin[0] == NULL)
+	    ui_delay(2000L, TRUE);
 	TIME_MSG("Warning delay");
     }
 
@@ -1135,6 +1186,10 @@ scripterror:
      * Set the default values for the options that use Rows and Columns.
      */
     ui_get_shellsize();		/* inits Rows and Columns */
+#ifdef FEAT_NETBEANS_INTG
+    if (usingNetbeans)
+	Columns += 2;		/* leave room for glyph gutter */
+#endif
     firstwin->w_height = Rows - p_ch;
     topframe->fr_height = Rows - p_ch;
 #ifdef FEAT_VERTSPLIT
@@ -1145,10 +1200,7 @@ scripterror:
     /* Set the 'diff' option now, so that it can be checked for in a .vimrc
      * file.  There is no buffer yet though. */
     if (diff_mode)
-    {
 	diff_win_options(firstwin, FALSE);
-	p_sbo = (char_u *)"ver,hor,jump";
-    }
 #endif
 
     cmdline_row = Rows - p_ch;
@@ -1853,6 +1905,12 @@ scripterror:
     if (restart_edit != 0)
 	stuffcharReadbuff(K_IGNORE);
 
+#ifdef FEAT_NETBEANS_INTG
+    if (usingNetbeans)
+	/* Tell the client that it can start sending commands. */
+	netbeans_startup_done();
+#endif
+
     TIME_MSG("before starting main loop");
 
     /*
@@ -1983,11 +2041,12 @@ main_loop(cmdwin)
 	    emsg_on_display = FALSE;	/* can delete error message now */
 	    did_emsg = FALSE;
 	    msg_didany = FALSE;		/* reset lines_left in msg_start() */
-	    do_redraw = FALSE;
 	    showruler(FALSE);
 
 	    setcursor();
 	    cursor_on();
+
+	    do_redraw = FALSE;
 	}
 #ifdef FEAT_GUI
 	if (need_mouse_correct)
@@ -2094,6 +2153,9 @@ getout(exitval)
 #if defined(USE_ICONV) && defined(DYNAMIC_ICONV)
     iconv_end();
 #endif
+#ifdef FEAT_NETBEANS_INTG
+    netbeans_end();
+#endif
 
     mch_exit(exitval);
 }
@@ -2303,10 +2365,10 @@ usage()
     main_msg(_("-dev <device>\t\tUse <device> for I/O"));
 #endif
 #ifdef FEAT_RIGHTLEFT
-    main_msg(_("-H\t\t\tstart in Hebrew mode"));
+    main_msg(_("-H\t\t\tStart in Hebrew mode"));
 #endif
 #ifdef FEAT_FKMAP
-    main_msg(_("-F\t\t\tstart in Farsi mode"));
+    main_msg(_("-F\t\t\tStart in Farsi mode"));
 #endif
     main_msg(_("-T <terminal>\tSet terminal type to <terminal>"));
     main_msg(_("-u <vimrc>\t\tUse <vimrc> instead of any .vimrc"));
@@ -2315,7 +2377,7 @@ usage()
 #endif
     main_msg(_("--noplugin\t\tDon't load plugin scripts"));
     main_msg(_("-o[N]\t\tOpen N windows (default: one for each file)"));
-    main_msg(_("-O[N]\t\tlike -o but split vertically"));
+    main_msg(_("-O[N]\t\tLike -o but split vertically"));
     main_msg(_("+\t\t\tStart at end of file"));
     main_msg(_("+<lnum>\t\tStart at line <lnum>"));
 #ifdef FEAT_PRECOMMANDS
@@ -2339,8 +2401,10 @@ usage()
 # endif
 #endif
 #ifdef FEAT_CLIENTSERVER
-    main_msg(_("--remote <files>\tEdit <files> in a Vim server and exit"));
+    main_msg(_("--remote <files>\tEdit <files> in a Vim server if possible"));
+    main_msg(_("--remote-silent <files>  Same, don't complain if there is no server"));
     main_msg(_("--remote-wait <files>  As --remote but wait for files to have been edited"));
+    main_msg(_("--remote-wait-silent <files>  Same, don't complain if there is no server"));
     main_msg(_("--remote-send <keys>\tSend <keys> to a Vim server and exit"));
     main_msg(_("--remote-expr <expr>\tEvaluate <expr> in a Vim server and print result"));
     main_msg(_("--serverlist\t\tList available Vim server names and exit"));
@@ -2349,8 +2413,8 @@ usage()
 #ifdef FEAT_VIMINFO
     main_msg(_("-i <viminfo>\t\tUse <viminfo> instead of .viminfo"));
 #endif
-    main_msg(_("-h\t\t\tprint Help (this message) and exit"));
-    main_msg(_("--version\t\tprint version information and exit"));
+    main_msg(_("-h\t\t\tPrint Help (this message) and exit"));
+    main_msg(_("--version\t\tPrint version information and exit"));
 
 #ifdef FEAT_GUI_X11
 # ifdef FEAT_GUI_MOTIF
@@ -2396,7 +2460,7 @@ usage()
     main_msg(_("--help\t\tShow Gnome arguments"));
 # endif
 #endif
-    mch_exit(1);
+    mch_exit(0);
 }
 
 #if defined(FEAT_GUI_DIALOG) || defined(FEAT_CON_DIALOG)
@@ -2543,6 +2607,7 @@ cmdsrv_main(argc, argv, serverName_arg, serverStr)
     char_u	*sname;
     int		ret;
     int		didone = FALSE;
+    int		exiterr = 0;
     char	**newArgV = argv + 1;
     int		newArgC = 1,
 		Argc = *argc;
@@ -2564,197 +2629,217 @@ cmdsrv_main(argc, argv, serverName_arg, serverStr)
     if (sname == NULL)
 	return;
 
-# ifdef FEAT_X11
-    if (xterm_dpy != NULL)	/* Win32 always works? */
-# endif
+    /*
+     * Execute the command server related arguments and remove them
+     * from the argc/argv array; We may have to return into main()
+     */
+    for (i = 1; i < Argc; i++)
     {
-	/*
-	 * Execute the command server related arguments and remove them
-	 * from the argc/argv array; We may have to return into main()
-	 */
-	for (i = 1; i < Argc; i++)
+	res = NULL;
+	if (STRCMP(argv[i], "--") == 0)	/* end of options */
 	{
-	    res = NULL;
-	    if (STRCMP(argv[i], "--") == 0)	/* end of options */
-	    {
-		for (; i < *argc; i++)
-		{
-		    *newArgV++ = argv[i];
-		    newArgC++;
-		}
-		break;
-	    }
-
-	    if (STRICMP(argv[i], "--remote") == 0)
-		argtype = ARGTYPE_EDIT;
-	    else if (STRICMP(argv[i], "--remote-silent") == 0)
-	    {
-		argtype = ARGTYPE_EDIT;
-		silent = TRUE;
-	    }
-	    else if (STRICMP(argv[i], "--remote-wait") == 0)
-		argtype = ARGTYPE_EDIT_WAIT;
-	    else if (STRICMP(argv[i], "--remote-wait-silent") == 0)
-	    {
-		argtype = ARGTYPE_EDIT_WAIT;
-		silent = TRUE;
-	    }
-	    else if (STRICMP(argv[i], "--remote-send") == 0)
-		argtype = ARGTYPE_SEND;
-	    else
-		argtype = ARGTYPE_OTHER;
-	    if (argtype != ARGTYPE_OTHER)
-	    {
-		if (i == *argc - 1)
-		    mainerr_arg_missing((char_u *)argv[i]);
-		if (argtype == ARGTYPE_SEND)
-		{
-		    *serverStr = (char_u *)argv[i + 1];
-		    i++;
-		}
-		else
-		{
-		    *serverStr = build_drop_cmd(*argc - i - 1, argv + i + 1,
-						argtype == ARGTYPE_EDIT_WAIT);
-		    if (*serverStr == NULL)
-		    {
-			/* Probably out of memory, exit. */
-			didone = TRUE;
-			break;
-		    }
-		    Argc = i;
-		}
-# ifdef FEAT_X11
-		ret = serverSendToVim(xterm_dpy, sname, *serverStr,
-						    NULL, &srv, 0, 0, silent);
-# else
-		ret = serverSendToVim(sname, *serverStr, NULL, &srv, 0, silent);
-# endif
-		if (ret < 0)
-		{
-		    if (argtype == ARGTYPE_SEND)
-		    {
-			/* Failed to send, abort. */
-			mch_errmsg(_("\nSend failed.\n"));
-			didone = TRUE;
-		    }
-		    else if (!silent)
-			/* Let vim start normally.  */
-			mch_errmsg(_("\nSend failed. Trying to execute locally\n"));
-		    break;
-		}
-
-# ifdef FEAT_GUI_W32
-		/* Guess that when the server name starts with "g" it's a GUI
-		 * server, which we can bring to the foreground here.
-		 * Foreground() in the server doesn't work very well. */
-		if (argtype != ARGTYPE_SEND && TO_UPPER(*sname) == 'G')
-		    SetForegroundWindow(srv);
-# endif
-
-		/*
-		 * For --remote-wait: Wait until the server did edit each
-		 * file.  Also detect that the server no longer runs.
-		 */
-		if (ret >= 0 && argtype == ARGTYPE_EDIT_WAIT)
-		{
-		    int	    numFiles = *argc - i - 1;
-		    int	    j;
-		    char_u  *done = alloc(numFiles);
-		    char_u  *p;
-# ifdef FEAT_GUI_W32
-		    NOTIFYICONDATA ni;
-		    int count = 0;
-		    extern HWND message_window;
-
-		    ni.cbSize = sizeof(ni);
-		    ni.hWnd = message_window;
-		    ni.uID = 0;
-		    ni.uFlags = NIF_ICON|NIF_TIP;
-		    ni.hIcon = LoadIcon((HINSTANCE)GetModuleHandle(0),
-								   "IDR_VIM");
-		    sprintf(ni.szTip, _("%d of %d edited"), count, numFiles);
-		    Shell_NotifyIcon(NIM_ADD, &ni);
-# endif
-
-		    /* Wait for all files to unload in remote */
-		    memset(done, 0, numFiles);
-		    while (memchr(done, 0, numFiles) != NULL)
-		    {
-# ifdef WIN32
-			p = serverGetReply(srv, NULL, TRUE, TRUE);
-			if (p == NULL)
-			    break;
-# else
-			if (serverReadReply(xterm_dpy, srv, &p, TRUE) < 0)
-			    break;
-# endif
-			j = atoi((char *)p);
-			if (j >= 0 && j < numFiles)
-			{
-# ifdef FEAT_GUI_W32
-			    ++count;
-			    sprintf(ni.szTip, _("%d of %d edited"),
-							     count, numFiles);
-			    Shell_NotifyIcon(NIM_MODIFY, &ni);
-# endif
-			    done[j] = 1;
-			}
-		    }
-# ifdef FEAT_GUI_W32
-		    Shell_NotifyIcon(NIM_DELETE, &ni);
-# endif
-		}
-	    }
-	    else if (STRICMP(argv[i], "--remote-expr") == 0)
-	    {
-		if (i == *argc - 1)
-		    mainerr_arg_missing((char_u *)argv[i]);
-# ifdef WIN32
-		if (serverSendToVim(sname, (char_u *)argv[i + 1],
-						    &res, NULL, 1, FALSE) < 0)
-# else
-		if (serverSendToVim(xterm_dpy, sname, (char_u *)argv[i + 1],
-						 &res, NULL, 1, 1, FALSE) < 0)
-# endif
-		    mch_errmsg(_("Send expression failed.\n"));
-	    }
-	    else if (STRICMP(argv[i], "--serverlist") == 0)
-	    {
-# ifdef WIN32
-		res = serverGetVimNames();
-#else
-		res = serverGetVimNames(xterm_dpy);
-#endif
-	    }
-	    else if (STRICMP(argv[i], "--servername") == 0)
-	    {
-		/* Alredy processed. Take it out of the command line */
-		i++;
-		continue;
-	    }
-	    else
+	    for (; i < *argc; i++)
 	    {
 		*newArgV++ = argv[i];
 		newArgC++;
-		continue;
 	    }
-	    didone = TRUE;
-	    if (res != NULL && *res != NUL)
-	    {
-		mch_msg((char *)res);
-		if (res[STRLEN(res) - 1] != '\n')
-		    mch_msg("\n");
-	    }
-	    vim_free(res);
+	    break;
 	}
 
-	if (didone)
+	if (STRICMP(argv[i], "--remote") == 0)
+	    argtype = ARGTYPE_EDIT;
+	else if (STRICMP(argv[i], "--remote-silent") == 0)
 	{
-	    display_errors();	/* display any collected messages */
-	    exit(0);		/* Mission accomplished - get out */
+	    argtype = ARGTYPE_EDIT;
+	    silent = TRUE;
 	}
+	else if (STRICMP(argv[i], "--remote-wait") == 0)
+	    argtype = ARGTYPE_EDIT_WAIT;
+	else if (STRICMP(argv[i], "--remote-wait-silent") == 0)
+	{
+	    argtype = ARGTYPE_EDIT_WAIT;
+	    silent = TRUE;
+	}
+	else if (STRICMP(argv[i], "--remote-send") == 0)
+	    argtype = ARGTYPE_SEND;
+	else
+	    argtype = ARGTYPE_OTHER;
+	if (argtype != ARGTYPE_OTHER)
+	{
+	    if (i == *argc - 1)
+		mainerr_arg_missing((char_u *)argv[i]);
+	    if (argtype == ARGTYPE_SEND)
+	    {
+		*serverStr = (char_u *)argv[i + 1];
+		i++;
+	    }
+	    else
+	    {
+		*serverStr = build_drop_cmd(*argc - i - 1, argv + i + 1,
+						argtype == ARGTYPE_EDIT_WAIT);
+		if (*serverStr == NULL)
+		{
+		    /* Probably out of memory, exit. */
+		    didone = TRUE;
+		    exiterr = 1;
+		    break;
+		}
+		Argc = i;
+	    }
+# ifdef FEAT_X11
+	    if (xterm_dpy == NULL)
+	    {
+		mch_errmsg(_("No display"));
+		ret = -1;
+	    }
+	    else
+		ret = serverSendToVim(xterm_dpy, sname, *serverStr,
+						    NULL, &srv, 0, 0, silent);
+# else
+	    /* Win32 always works? */
+	    ret = serverSendToVim(sname, *serverStr, NULL, &srv, 0, silent);
+# endif
+	    if (ret < 0)
+	    {
+		if (argtype == ARGTYPE_SEND)
+		{
+		    /* Failed to send, abort. */
+		    mch_errmsg(_(": Send failed.\n"));
+		    didone = TRUE;
+		    exiterr = 1;
+		}
+		else if (!silent)
+		    /* Let vim start normally.  */
+		    mch_errmsg(_(": Send failed. Trying to execute locally\n"));
+		break;
+	    }
+
+# ifdef FEAT_GUI_W32
+	    /* Guess that when the server name starts with "g" it's a GUI
+	     * server, which we can bring to the foreground here.
+	     * Foreground() in the server doesn't work very well. */
+	    if (argtype != ARGTYPE_SEND && TOUPPER_ASC(*sname) == 'G')
+		SetForegroundWindow(srv);
+# endif
+
+	    /*
+	     * For --remote-wait: Wait until the server did edit each
+	     * file.  Also detect that the server no longer runs.
+	     */
+	    if (ret >= 0 && argtype == ARGTYPE_EDIT_WAIT)
+	    {
+		int	numFiles = *argc - i - 1;
+		int	j;
+		char_u  *done = alloc(numFiles);
+		char_u  *p;
+# ifdef FEAT_GUI_W32
+		NOTIFYICONDATA ni;
+		int	count = 0;
+		extern HWND message_window;
+
+		ni.cbSize = sizeof(ni);
+		ni.hWnd = message_window;
+		ni.uID = 0;
+		ni.uFlags = NIF_ICON|NIF_TIP;
+		ni.hIcon = LoadIcon((HINSTANCE)GetModuleHandle(0), "IDR_VIM");
+		sprintf(ni.szTip, _("%d of %d edited"), count, numFiles);
+		Shell_NotifyIcon(NIM_ADD, &ni);
+# endif
+
+		/* Wait for all files to unload in remote */
+		memset(done, 0, numFiles);
+		while (memchr(done, 0, numFiles) != NULL)
+		{
+# ifdef WIN32
+		    p = serverGetReply(srv, NULL, TRUE, TRUE);
+		    if (p == NULL)
+			break;
+# else
+		    if (serverReadReply(xterm_dpy, srv, &p, TRUE) < 0)
+			break;
+# endif
+		    j = atoi((char *)p);
+		    if (j >= 0 && j < numFiles)
+		    {
+# ifdef FEAT_GUI_W32
+			++count;
+			sprintf(ni.szTip, _("%d of %d edited"),
+							     count, numFiles);
+			Shell_NotifyIcon(NIM_MODIFY, &ni);
+# endif
+			done[j] = 1;
+		    }
+		}
+# ifdef FEAT_GUI_W32
+		Shell_NotifyIcon(NIM_DELETE, &ni);
+# endif
+	    }
+	}
+	else if (STRICMP(argv[i], "--remote-expr") == 0)
+	{
+	    if (i == *argc - 1)
+		mainerr_arg_missing((char_u *)argv[i]);
+# ifdef WIN32
+	    /* Win32 always works? */
+	    if (serverSendToVim(sname, (char_u *)argv[i + 1],
+						    &res, NULL, 1, FALSE) < 0)
+# else
+	    if (xterm_dpy == NULL)
+		mch_errmsg(_("No display: Send expression failed.\n"));
+	    else if (serverSendToVim(xterm_dpy, sname, (char_u *)argv[i + 1],
+						 &res, NULL, 1, 1, FALSE) < 0)
+# endif
+	    {
+		if (res != NULL && *res != NUL)
+		{
+		    /* Output error from remote */
+		    mch_errmsg((char *)res);
+		    vim_free(res);
+		    res = NULL;
+		}
+		mch_errmsg(_(": Send expression failed.\n"));
+	    }
+	}
+	else if (STRICMP(argv[i], "--serverlist") == 0)
+	{
+# ifdef WIN32
+	    /* Win32 always works? */
+	    res = serverGetVimNames();
+# else
+	    if (xterm_dpy != NULL)
+		res = serverGetVimNames(xterm_dpy);
+# endif
+	    if (called_emsg)
+		mch_errmsg("\n");
+	}
+	else if (STRICMP(argv[i], "--servername") == 0)
+	{
+	    /* Alredy processed. Take it out of the command line */
+	    i++;
+	    continue;
+	}
+	else
+	{
+	    *newArgV++ = argv[i];
+	    newArgC++;
+	    continue;
+	}
+	didone = TRUE;
+	if (res != NULL && *res != NUL)
+	{
+	    mch_msg((char *)res);
+	    if (res[STRLEN(res) - 1] != '\n')
+		mch_msg("\n");
+	}
+	vim_free(res);
     }
+
+    if (didone)
+    {
+	display_errors();	/* display any collected messages */
+	exit(exiterr);	/* Mission accomplished - get out */
+    }
+
     /* Return back into main() */
     *argc = newArgC;
     vim_free(sname);
@@ -2791,7 +2876,8 @@ build_drop_cmd(filec, filev, sendReply)
     ga_init2(&ga, 1, 100);
     ga_concat(&ga, (char_u *)"<C-\\><C-N>:cd ");
     ga_concat(&ga, p);
-    ga_concat(&ga, (char_u *)"<CR>:drop");
+    /* Call inputsave() so that a prompt for an encryption key works. */
+    ga_concat(&ga, (char_u *)"<CR>:if exists('*inputsave')|call inputsave()|endif|drop");
     vim_free(p);
     for (i = 0; i < filec; i++)
     {
@@ -2814,7 +2900,10 @@ build_drop_cmd(filec, filev, sendReply)
 	ga_concat(&ga, p);
 	vim_free(p);
     }
-    ga_concat(&ga, (char_u *)"<CR>:cd -");
+    /* The :drop commands goes to Insert mode when 'insertmode' is set, use
+     * CTRL-\ CTRL-N again. */
+    ga_concat(&ga, (char_u *)"|if exists('*inputrestore')|call inputrestore()|endif<CR>");
+    ga_concat(&ga, (char_u *)"<C-\\><C-N>:cd -");
     if (sendReply)
 	ga_concat(&ga, (char_u *)"<CR>:call SetupRemoteReplies()");
     if (inicmd != NULL)
@@ -2822,8 +2911,9 @@ build_drop_cmd(filec, filev, sendReply)
 	ga_concat(&ga, (char_u *)"<CR>:");
 	ga_concat(&ga, inicmd);
     }
-    /* Bring the window to the foreground & clear command line */
-    ga_concat(&ga, (char_u *)"<CR>:call foreground()<CR>:<CR>");
+    /* Bring the window to the foreground, goto Insert mode when 'im' set and
+     * clear command line */
+    ga_concat(&ga, (char_u *)"<CR>:call foreground()<CR>:if &im|starti|endif|echo<CR>");
     ga_append(&ga, NUL);
     return ga.ga_data;
 }
@@ -2841,10 +2931,9 @@ server_to_input_buf(str)
     /* Set 'cpoptions' the way we want it.
      *    B set - backslashes are *not* treated specially
      *    k set - keycodes are *not* reverse-engineered
-     *    < unset - <Key> sequenecs *are* interpreted
-     *  last parameter of replace_termcodes() is TRUE so that
-     *  the <lt> sequence is recognised - needed as backslash
-     *  is not special...
+     *    < unset - <Key> sequences *are* interpreted
+     *  The last parameter of replace_termcodes() is TRUE so that the <lt>
+     *  sequence is recognised - needed for a real backslash.
      */
     p_cpo = (char_u *)"Bk";
     str = replace_termcodes((char_u *)str, &ptr, FALSE, TRUE);
