@@ -15,7 +15,7 @@
 
 #include "vim.h"
 
-#ifdef __QNX__
+#ifdef HAVE_STDARG_H
 # include <stdarg.h>
 #endif
 
@@ -204,7 +204,7 @@ msg_strtrunc(s)
 }
 
 /*
- * Truncate a string "s"  to "buf" with cell width "room".
+ * Truncate a string "s" to "buf" with cell width "room".
  * "s" and "buf" may be equal.
  */
     void
@@ -226,6 +226,12 @@ trunc_string(s, buf, room)
     /* First part: Start of the string. */
     for (e = 0; len < half; ++e)
     {
+	if (s[e] == NUL)
+	{
+	    /* text fits without truncating! */
+	    buf[e] = NUL;
+	    return;
+	}
 	n = ptr2cells(s + e);
 	if (len + n >= half)
 	    break;
@@ -288,7 +294,7 @@ trunc_string(s, buf, room)
  * shorter than IOSIZE!!!
  */
 #ifndef PROTO
-# ifndef __QNX__
+# ifndef HAVE_STDARG_H
 
 int
 #ifdef __BORLANDC__
@@ -329,9 +335,12 @@ smsg_attr(attr, s, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10)
     return msg_attr(IObuff, attr);
 }
 
-# else /* __QNX__ */
+# else /* HAVE_STDARG_H */
 
     int
+#ifdef __BORLANDC__
+_RTLENTRYF
+#endif
 smsg(char_u *s, ...)
 {
     va_list arglist;
@@ -343,6 +352,9 @@ smsg(char_u *s, ...)
 }
 
     int
+#ifdef __BORLANDC__
+_RTLENTRYF
+#endif
 smsg_attr(int attr, char_u *s, ...)
 {
     va_list arglist;
@@ -353,7 +365,7 @@ smsg_attr(int attr, char_u *s, ...)
     return msg_attr(IObuff, attr);
 }
 
-# endif /* __QNX__ */
+# endif /* HAVE_STDARG_H */
 #endif
 
 /*
@@ -615,7 +627,7 @@ add_msg_hist(s, len, attr)
 {
     struct msg_hist *p;
 
-    if (msg_hist_off)
+    if (msg_hist_off || msg_silent != 0)
 	return;
 
     /* Don't let the message history get too big */
@@ -705,11 +717,14 @@ msg_end_prompt()
  */
     void
 wait_return(redraw)
-    int	    redraw;
+    int		redraw;
 {
-    int		    c;
-    int		    oldState;
-    int		    tmpState;
+    int		c;
+    int		oldState;
+    int		tmpState;
+#ifndef ORG_HITRETURN
+    int		had_got_int;
+#endif
 
     if (redraw == TRUE)
 	must_redraw = CLEAR;
@@ -769,6 +784,9 @@ wait_return(redraw)
 #else
 	do
 	{
+	    /* Remember "got_int", if it is set vgetc() probably returns a
+	     * CTRL-C, but we need to loop then. */
+	    had_got_int = got_int;
 	    c = safe_vgetc();
 	    if (!global_busy)
 		got_int = FALSE;
@@ -782,7 +800,8 @@ wait_return(redraw)
 		c = K_IGNORE;
 	    }
 #endif
-	} while (c == Ctrl_C || c == K_IGNORE
+	} while ((had_got_int && c == Ctrl_C)
+				|| c == K_IGNORE
 #ifdef FEAT_GUI
 				|| c == K_VER_SCROLLBAR || c == K_HOR_SCROLLBAR
 #endif
@@ -792,9 +811,12 @@ wait_return(redraw)
 				|| c == K_RIGHTDRAG  || c == K_RIGHTRELEASE
 				|| c == K_MOUSEDOWN  || c == K_MOUSEUP
 				|| (!mouse_has(MOUSE_RETURN)
+				    && mouse_row != Rows - 1
 				    && (c == K_LEFTMOUSE
 					|| c == K_MIDDLEMOUSE
-					|| c == K_RIGHTMOUSE))
+					|| c == K_RIGHTMOUSE
+					|| c == K_X1MOUSE
+					|| c == K_X2MOUSE))
 #endif
 				);
 	ui_breakcheck();
@@ -802,11 +824,12 @@ wait_return(redraw)
 	/*
 	 * Avoid that the mouse-up event causes visual mode to start.
 	 */
-	if (c == K_LEFTMOUSE || c == K_MIDDLEMOUSE || c == K_RIGHTMOUSE)
+	if (c == K_LEFTMOUSE || c == K_MIDDLEMOUSE || c == K_RIGHTMOUSE
+					  || c == K_X1MOUSE || c == K_X2MOUSE)
 	    (void)jump_to_mouse(MOUSE_SETPOS, NULL, 0);
 	else
 #endif
-	    if (vim_strchr((char_u *)"\r\n ", c) == NULL)
+	    if (vim_strchr((char_u *)"\r\n ", c) == NULL && c != Ctrl_C)
 	{
 	    stuffcharReadbuff(c);
 	    do_redraw = TRUE;	    /* need a redraw even though there is
@@ -1162,10 +1185,11 @@ msg_make(arg)
  * the character/string -- webb
  */
     int
-msg_outtrans_special(str, from)
-    char_u	*str;
+msg_outtrans_special(strstart, from)
+    char_u	*strstart;
     int		from;	/* TRUE for lhs of a mapping */
 {
+    char_u	*str = strstart;
     int		retval = 0;
     char_u	*string;
     int		attr;
@@ -1174,7 +1198,14 @@ msg_outtrans_special(str, from)
     attr = hl_attr(HLF_8);
     while (*str != NUL)
     {
-	string = str2special(&str, from);
+	/* Leading and trailing spaces need to be displayed in <> form. */
+	if ((str == strstart || str[1] == NUL) && *str == ' ')
+	{
+	    string = (char_u *)"<Space>";
+	    ++str;
+	}
+	else
+	    string = str2special(&str, from);
 	len = vim_strsize(string);
 	/* Highlight special keys */
 	msg_puts_attr(string, len > 1
@@ -1740,6 +1771,7 @@ msg_puts_attr(s, attr)
 			/*FALLTHROUGH*/
 		    case ' ':		/* one extra page */
 		    case K_PAGEDOWN:
+		    case K_LEFTMOUSE:
 			lines_left = Rows - 1;
 			break;
 
@@ -2275,7 +2307,7 @@ do_dialog(type, title, message, buttons, dfltbutton, textfield)
 		    continue;
 		for (retval = 0; hotkeys[retval]; retval++)
 		{
-		    if (hotkeys[retval] == TO_LOWER(c))
+		    if (hotkeys[retval] == TOLOWER_LOC(c))
 			break;
 		}
 		if (hotkeys[retval])
@@ -2367,7 +2399,7 @@ msg_show_console_dialog(message, buttons, dfltbutton)
     p = confirm_msg + 1 + STRLEN(message);
     q = hotk;
     r = buttons;
-    *q = (char_u)TO_LOWER(*r);	/* define lowercase hotkey */
+    *q = (char_u)TOLOWER_LOC(*r);	/* define lowercase hotkey */
 
     /* Remember where the choices start, displaying starts here when "q" typed
      * at the more prompt. */
@@ -2380,7 +2412,7 @@ msg_show_console_dialog(message, buttons, dfltbutton)
 	{
 	    *p++ = ',';
 	    *p++ = ' ';	    /* '\n' -> ', ' */
-	    *(++q) = (char_u)TO_LOWER(*(r + 1)); /* next hotkey */
+	    *(++q) = (char_u)TOLOWER_LOC(*(r + 1)); /* next hotkey */
 	    if (dfltbutton)
 		--dfltbutton;
 	}
@@ -2395,7 +2427,7 @@ msg_show_console_dialog(message, buttons, dfltbutton)
 		*p++ = (dfltbutton == 1) ? '[' : '(';
 		*p++ = *r;
 		*p++ = (dfltbutton == 1) ? ']' : ')';
-		*q = (char_u)TO_LOWER(*r);	/* define lowercase hotkey */
+		*q = (char_u)TOLOWER_LOC(*r);	/* define lowercase hotkey */
 	    }
 	}
 	else
