@@ -402,7 +402,7 @@ qf_init(efile, errorformat, newlist)
 	}
 	*ptr++ = '$';
 	*ptr = NUL;
-	if ((fmt_ptr->prog = vim_regcomp(fmtstr, TRUE)) == NULL)
+	if ((fmt_ptr->prog = vim_regcomp(fmtstr, RE_MAGIC + RE_STRING)) == NULL)
 	    goto error2;
 	/*
 	 * Advance to next part
@@ -702,6 +702,9 @@ qf_get_fnum(directory, fname)
 	char_u	    *ptr;
 	int	    fnum;
 
+# ifdef VMS
+        vms_remove_version(fname);
+# endif
 	if (directory != NULL && !vim_isAbsName(fname)
 	    && (ptr = concat_fnames(directory, fname, TRUE)) != NULL)
 	{
@@ -1567,6 +1570,9 @@ ex_copen(eap)
 	win_goto(win);
     else
     {
+	/* The current window becomes the previous window afterwards. */
+	win = curwin;
+
 	/* Create the new window at the very bottom. */
 	win_goto(lastwin);
 	if (win_split(height, WSP_BELOW) == FAIL)
@@ -1592,6 +1598,9 @@ ex_copen(eap)
 	/* Only set the height when there is no window to the side. */
 	if (curwin->w_width == Columns)
 	    win_setheight(height);
+	curwin->w_p_wfh = TRUE;	    /* set 'winfixheight' */
+	if (win_valid(win))
+	    prevwin = win;
     }
 
     /*
@@ -1857,6 +1866,7 @@ buf_hide(buf)
     switch (buf->b_p_bh[0])
     {
 	case 'u':		    /* "unload" */
+	case 'w':		    /* "wipe" */
 	case 'd': return FALSE;	    /* "delete" */
 	case 'h': return TRUE;	    /* "hide" */
     }
@@ -2020,7 +2030,7 @@ ex_cfile(eap)
 {
     if (*eap->arg != NUL)
 	set_string_option_direct((char_u *)"ef", -1, eap->arg, OPT_FREE);
-    if (qf_init(p_ef, p_efm, TRUE) > 0)
+    if (qf_init(p_ef, p_efm, TRUE) > 0 && eap->cmdidx == CMD_cfile)
 	qf_jump(0, 0, eap->forceit);		/* display first error */
 }
 
